@@ -22,6 +22,15 @@
 #define CRYPTO_KEY_AUTH_GET	11
 #define CRYPTO_KEY_AUTH_DELETE	12
 
+/*
+ * HMAC_FILE_WRITE is normally the same key as HMAC_FILE, but can be set to
+ * a different value via a masked crypto_keys_import if we need to read from
+ * one archive set and write to another (i.e., tarsnap-recrypt).  Because it
+ * is a duplicate key, HMAC_FILE_WRITE cannot be exported or generated.
+ */
+/* Keys #13--18 reserved for server code. */
+#define CRYPTO_KEY_HMAC_FILE_WRITE 19
+
 /* Fake HMAC "key" to represent "just SHA256 the data". */
 #define CRYPTO_KEY_HMAC_SHA256	(-1)
 
@@ -35,6 +44,7 @@
 #define CRYPTO_KEYMASK_ENCR					\
 	(CRYPTO_KEYMASK_ENCR_PRIV | CRYPTO_KEYMASK_ENCR_PUB)
 #define CRYPTO_KEYMASK_HMAC_FILE	(1 << CRYPTO_KEY_HMAC_FILE)
+#define CRYPTO_KEYMASK_HMAC_FILE_WRITE	(1 << CRYPTO_KEY_HMAC_FILE_WRITE)
 #define CRYPTO_KEYMASK_HMAC_CHUNK	(1 << CRYPTO_KEY_HMAC_CHUNK)
 #define CRYPTO_KEYMASK_HMAC_NAME	(1 << CRYPTO_KEY_HMAC_NAME)
 #define CRYPTO_KEYMASK_HMAC_CPARAMS	(1 << CRYPTO_KEY_HMAC_CPARAMS)
@@ -75,7 +85,7 @@
 typedef struct crypto_session_internal CRYPTO_SESSION;
 
 /**
- * crypto_entropy_init():
+ * crypto_entropy_init(void):
  * Initialize the PRNG.
  */
 int crypto_entropy_init(void);
@@ -88,17 +98,18 @@ int crypto_entropy_init(void);
 int crypto_entropy_read(uint8_t *, size_t);
 
 /**
- * crypto_keys_init():
+ * crypto_keys_init(void):
  * Initialize the key cache.  Note that crypto_entropy_init MUST be called
  * before this function.
  */
 int crypto_keys_init(void);
 
 /**
- * crypto_keys_import(buf, buflen):
- * Import keys from the provided buffer into the key cache.
+ * crypto_keys_import(buf, buflen, keys):
+ * Import keys from the provided buffer into the key cache.  Ignore any keys
+ * not specified in the mask ${keys}.
  */
-int crypto_keys_import(uint8_t * buf, size_t buflen);
+int crypto_keys_import(uint8_t *, size_t, int);
 
 /**
  * crypto_keys_missing(keys):
@@ -136,7 +147,7 @@ void crypto_hash_data_key(const uint8_t *, size_t,
 
 /**
  * crypto_hash_data_key_2(key, keylen, data0, len0, data1, len1, buf):
- * Hash the concatentation of two buffers with the provided HMAC-SHA256 key.
+ * Hash the concatenation of two buffers with the provided HMAC-SHA256 key.
  */
 void crypto_hash_data_key_2(const uint8_t *, size_t,
     const uint8_t *, size_t, const uint8_t *, size_t, uint8_t[32]);
@@ -226,7 +237,7 @@ int crypto_dh_sanitycheck(const uint8_t[CRYPTO_DH_PUBLEN]);
 
 /**
  * crypto_session_init(pub, priv, nonce, mkey, encr_write, auth_write,
- *     encr_read, auth_read, proofs):
+ *     encr_read, auth_read):
  * Compute K = ${pub}^(2^258 + ${priv}), mkey = MGF1(nonce || K, 48), and
  * return a CRYPTO_SESSION with encryption and authentication write and read
  * keys constructed from HMAC(mkey, (encr|auth)_(write|read)).
