@@ -5,11 +5,11 @@
 #include <stdio.h>
 
 /**
- * keyfile_read(filename, machinenum):
+ * keyfile_read(filename, machinenum, keys):
  * Read keys from a tarsnap key file; and return the machine # via the
- * provided pointer.
+ * provided pointer.  Ignore any keys not specified in the ${keys} mask.
  */
-int keyfile_read(const char *, uint64_t *);
+int keyfile_read(const char *, uint64_t *, int);
 
 /**
  * keyfile_write(filename, machinenum, keys, passphrase, maxmem, cputime):
