@@ -9,6 +9,7 @@
 #include <string.h>
 
 #include "network_internal.h"
+#include "tvmath.h"
 #include "warnp.h"
 
 #include "network.h"
@@ -90,7 +91,8 @@ recalloc(void ** ptr, size_t clen, size_t nlen, size_t size)
 		cplen = nlen;
 	else
 		cplen = clen;
-	memcpy(nbuf, *ptr, cplen * size);
+	if (cplen > 0)
+		memcpy(nbuf, *ptr, cplen * size);
 
 	/* Zero any uninitialized buffer contents. */
 	memset((void *)((char *)nbuf + cplen * size), 0,
@@ -120,14 +122,24 @@ static int
 docallback(struct network_callback_internal * C, int timedout)
 {
 	network_callback * cb;
-	int status = NETWORK_STATUS_OK;
+	int status;
 
 	cb = C->callback;
 	C->callback = NULL;
-	if (timedout == -1)
+	switch (timedout) {
+	case -1:
 		status = NETWORK_STATUS_CANCEL;
-	else if (timedout == 1)
+		break;
+	case 0:
+		status = NETWORK_STATUS_OK;
+		break;
+	case 1:
 		status = NETWORK_STATUS_TIMEOUT;
+		break;
+	default:
+		warn0("Programmer error: Invalid status to docallback");
+		return (-1);
+	}
 
 	return ((cb)(C->cookie, status));
 }
@@ -195,7 +207,7 @@ selectstats_stopclock(void)
 }
 
 /**
- * network_init():
+ * network_init(void):
  * Initialize the network subsystem and return a cookie.
  */
 int
@@ -329,13 +341,14 @@ network_register(int fd, int op, struct timeval * timeo,
 	nci->cookie = cookie;
 
 	/* Convert the timeout into an absolute time. */
-	if (gettimeofday(&nci->timeout, NULL)) {
-		warnp("gettimeofday()");
+	if (timeo == NULL)
+		memset(&nci->timeout, 0, sizeof(struct timeval));
+	else
+		memcpy(&nci->timeout, timeo, sizeof(struct timeval));
+	if (tvmath_addctime(&nci->timeout)) {
 		nci->callback = NULL;
 		goto err0;
 	}
-	if (timeo != NULL)
-		tv_add(&nci->timeout, timeo);
 
 	/* Update the number of the highest used fd if necessary. */
 	if (N.maxfd < fd)
@@ -718,7 +731,7 @@ network_getselectstats(double * NN, double * mu, double * va, double * max)
 }
 
 /**
- * network_fini():
+ * network_fini(void):
  * Free resources associated with the network subsystem.
  */
 void
