@@ -40,14 +40,14 @@
 #define	HAVE_BZLIB_H 1
 #define	HAVE_CHFLAGS 1
 #define	HAVE_CHROOT 1
-#define HAVE_CLOCK_GETTIME 1
-#define HAVE_DECL_BE64ENC 1
+#define	HAVE_CLOCK_GETTIME 1
+#define	HAVE_DECL_BE64ENC 1
 #define	HAVE_DECL_OPTARG 1
 #define	HAVE_DECL_OPTIND 1
 #define	HAVE_DIRENT_D_NAMLEN 1
 #define	HAVE_DIRENT_H 1
 #define	HAVE_D_MD_ORDER 1
-#define HAVE_ERR_H 1
+#define	HAVE_ERR_H 1
 #define	HAVE_ERRNO_H 1
 #undef	HAVE_EXT2FS_EXT2_FS_H
 #define	HAVE_FCHDIR 1
@@ -82,7 +82,7 @@
 #endif
 #define	HAVE_PATHS_H 1
 #if __FreeBSD_version >= 700014
-#define HAVE_POSIX_MEMALIGN 1
+#define	HAVE_POSIX_MEMALIGN 1
 #endif
 #define	HAVE_PWD_H 1
 #define	HAVE_QSORT_R 1
@@ -101,12 +101,16 @@
 #define	HAVE_STRUCT_STAT_ST_FLAGS 1
 #undef	HAVE_STRUCT_STAT_ST_MTIM_TV_NSEC
 #define	HAVE_STRUCT_STAT_ST_MTIMESPEC_TV_NSEC 1
+#define	HAVE_STRUCT_STATFS_F_FSTYPENAME 1
+#define	HAVE_STRUCT_STATFS_F_TYPE 1
 #define	HAVE_SYS_ACL_H 1
 #define	HAVE_SYS_ENDIAN_H 1
 #define	HAVE_SYS_IOCTL_H 1
 #define	HAVE_SYS_PARAM_H 1
 #define	HAVE_SYS_STAT_H 1
-#define HAVE_SYSCTL_HW_USERMEM 1
+#define	HAVE_SYS_MOUNT_H 1
+#define	HAVE_SYS_STATVFS_H 1
+#define	HAVE_SYSCTL_HW_USERMEM 1
 #define	HAVE_TIME_H 1
 #define	HAVE_SYS_TYPES_H 1
 #define	HAVE_UINTMAX_T 1
